<?php
// $Id: sweaver_plugin_styles.inc,v 1.1.2.22 2010/09/13 21:34:49 swentel Exp $

/**
 * @file
 * Styles plugin.
 */
class sweaver_plugin_styles extends sweaver_plugin {

  /**
   * Menu registry.
   */
  function sweaver_menu(&$weight, $page_arguments, $base) {

    $items = array();
    $page_arguments = array(
      'plugin' => 'sweaver_plugin_styles',
    );

    // Styles administration.
    $items['admin/settings/sweaver/styles'] = $base + array(
      'title' => 'Styles',
      'page arguments' => array($page_arguments),
      'type' => MENU_LOCAL_TASK,
      'weight' => $weight++,
    );

    // Delete a style.
    $page_arguments['callback_method'] = 'style_confirm_form_delete';
    $items['admin/settings/sweaver/styles/delete'] = $base + array(
      'title' => 'Delete style',
      'page arguments' => array($page_arguments),
      'type' => MENU_CALLBACK,
    );

    return $items;
  }

  /**
   * Frontend form: add styles form.
   */
  function sweaver_form() {
    $form = array();
    $form['#style_containers'] = array();
    global $theme_key;

    // Current style.
    $current_style = $this->sweaver_get_current_style($theme_key);

    // Saved styles in database.
    $existing_styles = FALSE;
    $existing_styles_options = array();
    $results = db_query("SELECT style_id, style FROM {sweaver_style_draft} where theme = '%s'", $theme_key);
    while ($data = db_fetch_array($results)) {
      $existing_styles_options[$data['style_id']] = $data['style'];
    }
    if (count($existing_styles_options) > 0) {
      $existing_styles = TRUE;
    }

    // Add save to containers.
    $form['#style_containers']['save_style_form'] = array(
      'container_name' => t('Save style'),
      'properties' => array(
        'radio_save_type',
        'radio_draft_type',
        'radio_style_save_type',
        'new_style_name',
        'existing_style_id',
        'save_submit',
      ),
    );

    // Save style form.
    $form['radio_save_type'] = array(
      '#type' => 'select',
      '#options' => array(
        1 => t('Save as active style'),
        0 => t('Save as draft'),
      ),
      '#default_value' => 0,
      '#attributes' => array('class' => 'radio-save-type'),
    );
    $form['radio_draft_type'] = array(
      '#type' => 'select',
      '#options' => array(
        0 => t('Leave draft mode'),
        1 => t('Stay in draft mode'),
      ),
      '#default_value' => 1,
      '#prefix' => '<div class="styles-draft-type">',
      '#suffix' => '</div>',
    );

    // New style name.
    $form['new_style_name'] = array(
      '#type' => 'textfield',
      '#size' => 50,
    );

    // Existing styles.
    if ($existing_styles) {

      // Hide new style name by default.
      $form['new_style_name']['#attributes'] = array('class' => 'display-none');

      $form['radio_style_save_type'] = array(
        '#type' => 'select',
        '#options' => array(
          0 => t('New style'),
          1 => t('Existing style'),
        ),
        '#default_value' => 1,
        '#attributes' => array('class' => 'radio-style-save-type'),
      );

      $form['existing_style_id'] = array(
        '#type' => 'select',
        '#options' => $existing_styles_options,
        '#default_value' => (isset($current_style->style_id)) ? $current_style->style_id : 0,
      );
    }
    else {
      // If no existing styles, set style save type to new.
      $form['radio_style_save_type'] = array(
        '#type' => 'value',
        '#value' => 0,
      );
    }

    $form['save_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save style'),
    );

    // Load / delete style form.
    if ($existing_styles) {

      // Add load and delete to containers.
      $form['#style_containers']['load_style_form'] = array(
        'container_name' => t('Load style'),
        'properties' => array(
          'load_style',
          'load_active',
          'load_submit',
        ),
      );
      $form['#style_containers']['delete_style_form'] = array(
        'container_name' => t('Delete style'),
        'properties' => array(
          'delete_style',
          'delete_submit',
        ),
      );

      $form['load_style'] = array(
        '#type' => 'select',
        '#title' => t('Style'),
        '#options' => $existing_styles_options,
      );
      $form['load_active'] = array(
        '#type' => 'checkbox',
        '#title' => t('Set as active style'),
        '#default_value' => FALSE,
      );
      $form['load_submit'] = array(
        '#type' => 'submit',
        '#value' => t('Load style'),
      );

      $form['delete_style'] = array(
        '#type' => 'select',
        '#title' => t('Style'),
        '#options' => $existing_styles_options,
      );
      $form['delete_submit'] = array(
        '#type' => 'submit',
        '#value' => t('Delete style'),
      );
    }

    return $form;
  }

  /**
   * Frontend form render.
   */
  function sweaver_form_render(&$vars, &$form, $plugin) {

    $name = $plugin['name'];
    $vars['tabs'][$name]['#tab_name'] = $form[$name]['#tab_name'];
    $vars['tabs_data'][$name]['#tab_description'] = $form[$name]['#tab_description'];

    // Containers.
    $output = '';
    $container_count = 1;
    $container_total = count($form[$name]['form']['#style_containers']);

    foreach ($form[$name]['form']['#style_containers'] as $key => $container_value) {
      // Add first/last classes to the containers.
      $container_class = ' container-'. $container_total;
      if ($container_count == 1) {
        $container_class .= ' container-first';
      }
      if ($container_total == $container_count) {
        $container_class .= ' container-last';
      }
      $output .= '<div class="container '. $container_class .'"><div class="container-inner">';
      $output .= '<h2>'. $container_value['container_name'] .'</h2>';
      foreach ($container_value['properties'] as $key => $form_element) {
        if (isset($form[$name]['form'][$form_element])) {
          $output .= drupal_render($form[$name]['form'][$form_element]);
        }
      }
      $output .= '</div></div>';
      $container_count++;
    }

    $vars['tabs_data'][$name]['content'] = $output;
  }

  /**
   * Frontend css and js.
   */
  function sweaver_form_css_js(&$inline_settings) {
    drupal_add_js(drupal_get_path('module', 'sweaver') .'/plugins/sweaver_plugin_styles/sweaver_plugin_styles.js', 'module');
  }

  /**
   * Frontend form submit.
   */
  function sweaver_form_submit($form, &$form_state) {

    // Save style.
    if ($form_state['clicked_button']['#value'] == t('Save style')) {
      $update = array();
      $style = new stdClass;
      $theme_key = $form['#current_theme'];

      // Create variables from form_state, easier to re-use theme in the flow.
      $radio_save_type = $form_state['values']['radio_save_type'];
      $radio_draft_type = $form_state['values']['radio_draft_type'];
      $radio_style_save_type = $form_state['values']['radio_style_save_type'];
      $new_style_name = $form_state['values']['new_style_name'];
      $existing_style_id = isset($form_state['values']['existing_style_id']) ? $form_state['values']['existing_style_id'] : FALSE;

      if ($radio_style_save_type) {
        $update = array('style_id');
        $style->style_id = $existing_style_id;
        $style->style = $form['sweaver_plugin_styles']['form']['existing_style_id']['#options'][$style->style_id];
      }
      else {
        $style->style = $new_style_name;
        if (empty($style->style)) {
          $this->sweaver_session(t('You need to enter a name for your style.'));
          return;
        }
      }

      // Save the draft version.
      $style->theme = $theme_key;
      $style->css = $form_state['values']['css'];
      drupal_write_record('sweaver_style_draft', $style, $update);
      $this->sweaver_export_file($form_state['values']['css_rendered'], $style);
      $form_state['saved_style_id'] = $style->style_id;
      $form_state['save_live'] = $radio_save_type;

      // Set active.
      if ($radio_save_type) {
        $style->active = 1;
        $this->sweaver_session(NULL, 'draft_mode', TRUE);
        $this->sweaver_session(NULL, 'loaded_style', TRUE);
        $this->sweaver_export_file($form_state['values']['css_rendered'], $style, 'live');
        $message = 'The style @style has been saved and set active.';
        db_query("UPDATE {sweaver_style} set active = 0 WHERE theme = '%s'", $style->theme);
        // Find out first if this style already exists or not.
        if (db_result(db_query("SELECT style_id FROM {sweaver_style} WHERE style_id = %d AND theme = '%s'", $style->style_id, $style->theme)) === FALSE) {
          $update = array();
        }
        drupal_write_record('sweaver_style', $style, $update);
      }
      // Save as draft
      else {
        // Keep in draft mode.
        if ($radio_draft_type) {
          $this->sweaver_session(TRUE, 'draft_mode');
          $this->sweaver_session($style->style_id, 'loaded_style');
          $message = 'The style @style has been saved and you are now working in draft mode.';
        }
        // Leave draft mode.
        else {
          $this->sweaver_session(NULL, 'draft_mode', TRUE);
          $this->sweaver_session(NULL, 'loaded_style', TRUE);
          $message = 'The style @style has been saved and you left draft mode.';
        }
      }

      $this->sweaver_session(t($message, array('@style' => $style->style)));
    }

    // Load style.
    if ($form_state['clicked_button']['#value'] == t('Load style')) {
      $this->sweaver_session(NULL, 'draft_mode', TRUE);
      $this->sweaver_session(NULL, 'loaded_style', TRUE);

      $theme_key = $form['#current_theme'];
      $style_id = $form_state['values']['load_style'];
      $active = $form_state['values']['load_active'];
      $style_name = $form['sweaver_plugin_styles']['form']['load_style']['#options'][$style_id];

      if (!$active) {
        $this->sweaver_session(TRUE, 'draft_mode');
        $this->sweaver_session($style_id, 'loaded_style');
        $this->sweaver_session(t('The style @style has been loaded and you are now working in draft mode.', array('@style' => $style_name)));
      }
      else {
        $this->sweaver_session(t('The style @style has been loaded and set active.', array('@style' => $style_name)));
        db_query("UPDATE {sweaver_style} set active = 0 WHERE theme = '%s'", $theme_key);
        $style = db_fetch_object(db_query("SELECT * FROM {sweaver_style_draft} WHERE style_id = %d", $style_id));
        $style->active = 1;
        // Find out first if this style already exists or not.
        if (db_result(db_query("SELECT style_id FROM {sweaver_style} WHERE style_id = %d AND theme = '%s'", $style->style_id, $style->theme)) === FALSE) {
          $update = array();
        }
        else {
          $update = array('style_id');
        }
        drupal_write_record('sweaver_style', $style, $update);
        $form_state['loaded_style'] = $style;

        // Copy draft css to live.
        $draft = file_directory_path() .'/sweaver/sweaver_'. $style->theme . '_'. $style->style_id .'_draft.css';
        $live = file_directory_path() .'/sweaver/sweaver_'. $style->theme . '_'. $style->style_id .'_live.css';
        file_copy($draft, $live, FILE_EXISTS_REPLACE);
      }
    }

    // Delete style.
    if ($form_state['clicked_button']['#value'] == t('Delete style')) {

      // Reset session.
      $this->sweaver_session(NULL, 'draft_mode', TRUE);
      $this->sweaver_session(NULL, 'loaded_style', TRUE);

      // Get info from db and store in form_state so other modules can profit from it.
      $style_id = $form_state['values']['delete_style'];
      $style = db_fetch_object(db_query("SELECT * FROM {sweaver_style_draft} WHERE style_id = %d", $style_id));
      $style_live = db_fetch_object(db_query("SELECT * FROM {sweaver_style} WHERE style_id = %d", $style_id));
      $form_state['style_active'] = isset($style_live->active) ? $style_live->active : FALSE;
      $form_state['style_to_delete'] = $style;

      // Delete entries from tables, files.
      db_query("DELETE FROM {sweaver_style} WHERE style_id = %d", $style_id);
      db_query("DELETE FROM {sweaver_style_draft} WHERE style_id = %d", $style_id);
      $style_name = $form['sweaver_plugin_styles']['form']['delete_style']['#options'][$style_id];
      $draft = file_directory_path() .'/sweaver/sweaver_'. $style->theme .'_'. $style->style_id .'_draft.css';
      $live = file_directory_path() .'/sweaver/sweaver_'. $style->theme .'_'. $style->style_id .'_live.css';
      file_delete($draft);
      file_delete($live);

      // Message.
      $this->sweaver_session(t('The style @style has been deleted.', array('@style' => $style_name)));
    }

    // Clear page & block cache.
    cache_clear_all();
  }

  /**
   * Export css to file.
   */
  function sweaver_export_file($css, $style, $type = 'draft') {
    $css = str_replace('<style type="text/css">', '', $css);
    $css = str_replace('</style>', '', $css);
    $filename = 'sweaver_' . $style->theme . '_'. $style->style_id .'_'. $type .'.css';

    // Create the css within the files folder.
    $path = file_create_path('sweaver');
    file_check_directory($path, FILE_CREATE_DIRECTORY);
    $file = $path .'/'. $filename;
    if (!$fp = fopen($file, 'w')) {
      $this->sweaver_session(t('The css file could not be created.'));
    }
    else {
      // Clean the css before writing it.
      // Check for special case where font family names are enclosed in quotations,
      // we don't want html entities in the css.
      $css = check_plain($css);
      $css = str_replace('&#039;', "'", $css);
      fwrite($fp, $css);
    }
    @fclose($fp);
  }

  /**
   * Menu callback
   */
  function menu_callback() {
    $form = array();

    // Settings.
    $form['sweaver_load_style_when_editor_inactive'] = array(
      '#type' => 'checkbox',
      '#title' => t('Load style'),
      '#description' => t('Load active style for the theme when the editor is inactive. You can also add a line to the info file of your theme like this: stylesheets[all][] = /path/to/cssfile after you copied the file to your theme path. If you do that, you can ignore this setting completely and even disable this module after you have themed your website.'),
      '#default_value' => variable_get('sweaver_load_style_when_editor_inactive', TRUE),
    );

    $form = system_settings_form($form);

    // Styles list.
    $rows = array();
    $result = db_query("SELECT ssd.*, ss.active FROM {sweaver_style_draft} ssd LEFT JOIN {sweaver_style} ss on ss.style_id = ssd.style_id ORDER BY ssd.style ASC, ssd.theme ASC, ss.active DESC");
    while ($style = db_fetch_object($result)) {
      $row = array();
      $row[] = check_plain($style->style);
      $row[] = check_plain($style->theme);
      $row[] = ($style->active) ? t('Active') : t('Inactive');
      $operations = l(t('Delete'), 'admin/settings/sweaver/styles/delete/'. $style->style_id);
      $row[] = $operations;
      $rows[] = $row;
    }

    if (empty($rows)) {
      $output = '<p>'. t('No styles found.') .'</p>';
    }
    else {

      $header = array(
        t('Style'),
        t('Theme'),
        t('Status'),
        t('Operations'),
      );

      // Styles list.
      $output = theme('table', $header, $rows);

    }

    $form['item'] = array(
      '#type' => 'item',
      '#value' => $output,
    );

    return $form;
  }

  /**
   * Menu callback, delete style.
   */
  function style_confirm_form_delete() {
    $style_id = arg(5);
    $style = db_fetch_object(db_query('SELECT * FROM {sweaver_style} WHERE style_id = %d', $style_id));
    if ($style->style_id) {
      $form['#style'] = $style;
      return confirm_form($form, t('Are you sure you want to delete style %style?', array('%style' => $style->style)), 'admin/settings/sweaver/styles');
    }
    else {
      drupal_set_message(t('Style not found'));
      drupal_goto('admin/settings/sweaver/styles');
    }
  }

  /**
   * Submit callback, delete style.
   */
  function style_confirm_form_delete_submit($form, &$form_state) {
    $style = $form['#style'];
    db_query("DELETE FROM {sweaver_style} WHERE style_id = %d", $style->style_id);
    db_query("DELETE FROM {sweaver_style_draft} WHERE style_id = %d", $style->style_id);
    $this->sweaver_delete_file($style);
    cache_clear_all();
    drupal_set_message(t('Style %style has been removed', array('%style' => $form['#style']->style)));
    $form_state['redirect'] = 'admin/settings/sweaver/styles';
  }
}
